#include "matrix.h"
#include "mexConversion.h"
#include "heap.h"
#include <math.h>

void addNeighbors2Heap(MATRIX3 unvisited, HEAP *h, MATRIXD yavg, int currX, int currY, int currLabel, int objType){
    if (objType == 0){
        if (currX - 1 >= 0 && unvisited.data[currX - 1][currY][objType]){
            insertHeap(h, currX - 1, currY, currLabel, 1 - yavg.data[currX - 1][currY]);
            unvisited.data[currX - 1][currY][objType] = 0;
        }
        if (currX + 1 < unvisited.d1 && unvisited.data[currX + 1][currY][objType]){
            insertHeap(h, currX + 1, currY, currLabel, 1 - yavg.data[currX + 1][currY]);
            unvisited.data[currX + 1][currY][objType] = 0;
        }
        if (currY - 1 >= 0 && unvisited.data[currX][currY - 1][objType]){
            insertHeap(h, currX, currY - 1, currLabel, 1 - yavg.data[currX][currY - 1]);
            unvisited.data[currX][currY - 1][objType] = 0;
        }
        if (currY + 1 < unvisited.d2 && unvisited.data[currX][currY + 1][objType]){
            insertHeap(h, currX, currY + 1, currLabel, 1 - yavg.data[currX][currY + 1]);
            unvisited.data[currX][currY + 1][objType] = 0;
        }
    }
    else{
        if (currX - 1 >= 0 && unvisited.data[currX - 1][currY][objType]){
            insertHeap(h, currX - 1, currY, currLabel, yavg.data[currX - 1][currY]);
            unvisited.data[currX - 1][currY][objType] = 0;
        }
        if (currX + 1 < unvisited.d1 && unvisited.data[currX + 1][currY][objType]){
            insertHeap(h, currX + 1, currY, currLabel, yavg.data[currX + 1][currY]);
            unvisited.data[currX + 1][currY][objType] = 0;
        }
        if (currY - 1 >= 0 && unvisited.data[currX][currY - 1][objType]){
            insertHeap(h, currX, currY - 1, currLabel, yavg.data[currX][currY - 1]);
            unvisited.data[currX][currY - 1][objType] = 0;
        }
        if (currY + 1 < unvisited.d2 && unvisited.data[currX][currY + 1][objType]){
            insertHeap(h, currX, currY + 1, currLabel, yavg.data[currX][currY + 1]);
            unvisited.data[currX][currY + 1][objType] = 0;
        }
    }
}
MATRIX3 initializeUncertain(MATRIX obj, int *uncertainNo){
    MATRIX3 unvisited = allocateMatrix3(obj.row, obj.column, 2);
    int i, j;
    
    initializeMatrix3(&unvisited, 0);
    *uncertainNo = 0;
    for (i = 0; i < obj.row; i++)
        for (j = 0; j < obj.column; j++)
            if (obj.data[i][j] == 0){
                unvisited.data[i][j][0] = 1;
                unvisited.data[i][j][1] = 1;
                (*uncertainNo)++;
            }
    return unvisited;
}
void regionGrow(MATRIX obj, MATRIX objTypes, MATRIXD yavg){
    int uncertainNo, i, j, currX, currY, currLabel;
    MATRIX3 unvisited = initializeUncertain(obj, &uncertainNo);
    HEAP h = initializeHeap(uncertainNo * 2, MAXHEAP);
    HEAPDATA curr;
    int temp = 0;
    
    for (i = 0; i < obj.row; i++)
        for (j = 0; j < obj.column; j++)
            if (obj.data[i][j])
                addNeighbors2Heap(unvisited, &h, yavg, i, j, obj.data[i][j], objTypes.data[obj.data[i][j]][0]);
    
    while (h.size > 0){
        deleteHeap(&h, &curr);
        currX = curr.x;
        currY = curr.y;
        if (obj.data[currX][currY] == 0){
            currLabel = curr.label;
            obj.data[currX][currY] = currLabel;
            unvisited.data[currX][currY][0] = 0;
            unvisited.data[currX][currY][1] = 0;
            addNeighbors2Heap(unvisited, &h, yavg, currX, currY, currLabel, objTypes.data[obj.data[currX][currY]][0]);
            uncertainNo--;
        }
        temp++;
    }
    freeHeap(h);
    freeMatrix3(unvisited);
}
void excludeBackgroundObjects(MATRIX obj, MATRIX objTypes){
    int i, j;
    for (i = 0; i < obj.row; i++)
        for (j = 0; j < obj.column; j++){
            if (obj.data[i][j] == 0)
                mexPrintf("Something wrong (excludeBackgroundObjects)\n");
            else if (objTypes.data[obj.data[i][j]][0] == 0)
                obj.data[i][j] = 0;
        }
}
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){
    MATRIX obj      = convertMxArray2Matrix(prhs[0]);
	MATRIX objTypes = convertMxArray2Matrix(prhs[1]);
    MATRIXD yavg    = convertMxArray2MatrixD(prhs[2]);
    
    regionGrow(obj, objTypes, yavg);
    excludeBackgroundObjects(obj, objTypes);
    plhs[0] = convertMatrix2MxArray(obj);
	
    freeMatrix(obj);
    freeMatrix(objTypes);
	freeMatrixD(yavg);
}
