#include <stdio.h>
#include <stdlib.h>
#include "heap.h"

HEAP initializeHeap(int maxSize, int heapType){
	HEAP H;
	H.heapType = heapType;
	H.size = 0;
	H.maxSize = maxSize;
	H.data = (HEAPDATA *)calloc(maxSize, sizeof(HEAPDATA));
	return H;
}
void freeHeap(HEAP H){
	free(H.data);
}
int insertHeap(HEAP *H, int x, int y, int label, double key){
	if (H->heapType == MINHEAP)
		return insertMinHeap(H, x, y, label, key);
	else
		return insertMaxHeap(H, x, y, label, key);
}
int deleteHeap(HEAP *H, HEAPDATA *item){
	if (H->heapType == MINHEAP)
		return deleteMinHeap(H, item);
	else
		return deleteMaxHeap(H, item);
}
int insertMaxHeap(HEAP *H, int x, int y, int label, double key){
	int place, parent;
	HEAPDATA tmp;
	
    if (H->size >= H->maxSize)
        return 0;
	
	H->data[H->size].key = key;
	H->data[H->size].x = x;
	H->data[H->size].y = y;
	H->data[H->size].label = label;
    
	place = H->size;
	parent = (place - 1) / 2;
	
	while (place > 0){
		if (H->data[place].key < H->data[parent].key)
			break;
        
		tmp = H->data[place];
		H->data[place] = H->data[parent];
		H->data[parent] = tmp;
		
		place = parent;
		parent = (place - 1) / 2;
	}
	H->size += 1;
    return 1;
}
int deleteMaxHeap(HEAP *H, HEAPDATA *item){
	HEAPDATA tmp;
	int root, child, rchild;
	
	if (H->size == 0)
        return 0;
	
	*item = H->data[0];
	H->size -= 1;
	H->data[0] = H->data[H->size];
	
	root = 0;
	child = 2 * root + 1; // left child, if any
	while (child < H->size){
		rchild = child + 1; // right child
		if (rchild < H->size){
			if (H->data[rchild].key > H->data[child].key)
				child = rchild;
		}
		if (H->data[root].key > H->data[child].key)
			break;
		
		tmp = H->data[root];
		H->data[root] = H->data[child];
		H->data[child] = tmp;
		root = child;
		child = 2 * root + 1;
	}
	return 1;
}
int insertMinHeap(HEAP *H, int x, int y, int label, double key){
	int place, parent;
	HEAPDATA tmp;
	
    if (H->size >= H->maxSize)
        return 0;
	
	H->data[H->size].key = key;
	H->data[H->size].x = x;
	H->data[H->size].y = y;
	H->data[H->size].label = label;
	
	place = H->size;
	parent = (place - 1) / 2;
	
	while (place > 0){
		if (H->data[place].key > H->data[parent].key)
		break;
		
		tmp = H->data[place];
		H->data[place] = H->data[parent];
		H->data[parent] = tmp;
		
		place = parent;
		parent = (place - 1) / 2;
	}
	H->size += 1;
	return 1;
}
int deleteMinHeap(HEAP *H, HEAPDATA *item){
	HEAPDATA tmp;
	int root, child, rchild;
	
	if (H->size == 0)
		return 0;
	
	*item = H->data[0];
	H->size -= 1;
	H->data[0] = H->data[H->size];
	
	root = 0;
	child = 2 * root + 1; // left child, if any
	while (child < H->size){
		rchild = child + 1; // right child
		if (rchild < H->size){
			if (H->data[rchild].key < H->data[child].key)
			child = rchild;
		}
		if (H->data[root].key < H->data[child].key)
		break;
		
		tmp = H->data[root];
		H->data[root] = H->data[child];
		H->data[child] = tmp;
		root = child;
		child = 2 * root + 1;
	}
	return 1;
}




